@@ -1,10 +1,10 @@
-(* $Id: pxp_dtd.ml,v 1.21 2002/03/10 23:39:28 gerd Exp $
+(* $Id: pxp_dtd.ml,v 1.25 2003/06/20 15:14:13 gerd Exp $
  * ----------------------------------------------------------------------
  * PXP: The polymorphic XML parser for Objective Caml.
  * Copyright by Gerd Stolpmann. See LICENSE for details.
  *)
 
-open Pxp_types
+open Pxp_core_types
 open Pxp_lexer_types
 open Pxp_lexers
 open Pxp_entity
@@ -98,13 +98,14 @@ object (self)
 ;;
 
 
-class dtd  the_warner init_encoding =
+class dtd  ?swarner the_warner init_encoding =
   object (self)
     val mutable root = (None : string option)
     val mutable id =   (None : dtd_id option)
     val mutable mng =  (None : namespace_manager option)
 
     val warner       = (the_warner : collect_warnings)
+    val swarner      = (swarner : symbolic_warnings option)
     val encoding     = init_encoding
     val lexerset     = Pxp_lexers.get_lexer_set init_encoding
 
@@ -127,19 +128,19 @@ class dtd  the_warner init_encoding =
     initializer
     let w = new drop_warnings in
     self # add_gen_entity 
-      (new internal_entity self "lt"   w "&#38;#60;" false false encoding)
+      (new internal_entity self "lt"   None w "&#38;#60;" false false encoding)
       false;
     self # add_gen_entity 
-      (new internal_entity self "gt"   w "&#62;"     false false encoding)
+      (new internal_entity self "gt"   None w "&#62;"     false false encoding)
       false;
     self # add_gen_entity 
-      (new internal_entity self "amp"  w "&#38;#38;" false false encoding)
+      (new internal_entity self "amp"  None w "&#38;#38;" false false encoding)
       false;
     self # add_gen_entity 
-      (new internal_entity self "apos" w "&#39;"     false false encoding)
+      (new internal_entity self "apos" None w "&#39;"     false false encoding)
       false;
     self # add_gen_entity 
-      (new internal_entity self "quot" w "&#34;"     false false encoding)
+      (new internal_entity self "quot" None w "&#34;"     false false encoding)
       false;
 
 
@@ -147,6 +148,8 @@ class dtd  the_warner init_encoding =
 
     method warner = warner
 
+    method swarner = swarner
+
     method set_root r =
       if root = None then
 	root <- Some r
@@ -189,7 +192,7 @@ class dtd  the_warner init_encoding =
       (* raises Not_found if 'el' has already been added *)
       (* Note: 'el' is encoded in the same way as 'self'! *)
       let name = el # name in
-      check_name warner name;
+      check_name ?swarner warner name;
       if Str_hashtbl.mem elements name then
 	raise Not_found;
       Str_hashtbl.add elements name el;
@@ -207,7 +210,7 @@ class dtd  the_warner init_encoding =
       if en # encoding <> encoding then
 	failwith "Pxp_dtd.dtd # add_gen_entity: Inconsistent encodings";
       let name = en # name in
-      check_name warner name;
+      check_name ?swarner warner name;
       if Str_hashtbl.mem gen_entities name then begin
 	if List.mem name [ "lt"; "gt"; "amp"; "quot"; "apos" ] then begin
 	  (* These are allowed to be declared several times *)
@@ -231,7 +234,7 @@ class dtd  the_warner init_encoding =
 					"' redeclared"))
 	end
 	else
-	  warner # warn ("Entity `" ^ name ^ "' declared twice")
+	  warn swarner warner (`W_entity_declared_twice name)
       end
       else begin
 	Str_hashtbl.add gen_entities name (en, extdecl);
@@ -243,13 +246,13 @@ class dtd  the_warner init_encoding =
       if en # encoding <> encoding then
 	failwith "Pxp_dtd.dtd # add_par_entity: Inconsistent encodings";
       let name = en # name in
-      check_name warner name;
+      check_name ?swarner warner name;
       if not (Str_hashtbl.mem par_entities name) then begin
 	Str_hashtbl.add par_entities name en;
 	par_entity_names <- name :: par_entity_names
       end
       else
-	warner # warn ("Entity `" ^ name ^ "' declared twice")
+	warn swarner warner (`W_entity_declared_twice name)
 
 
     method add_notation no =
@@ -257,7 +260,7 @@ class dtd  the_warner init_encoding =
       if no # encoding <> encoding then
 	failwith "Pxp_dtd.dtd # add_notation: Inconsistent encodings";
       let name = no # name in
-      check_name warner name;
+      check_name ?swarner warner name;
       if Str_hashtbl.mem notations name then
 	raise (Validation_error("Notation `" ^ name ^ "' declared twice"));
       Str_hashtbl.add notations name no;
@@ -268,7 +271,7 @@ class dtd  the_warner init_encoding =
       if pi # encoding <> encoding then
 	failwith "Pxp_dtd.dtd # add_pinstr: Inconsistent encodings";
       let name = pi # target in
-      check_name warner name;
+      check_name ?swarner warner name;
 
       if String.length name >= 4 && String.sub name 0 4 = "pxp:" then begin
 	match name with
@@ -327,7 +330,7 @@ class dtd  the_warner init_encoding =
       end;
       Str_hashtbl.add pinstr name pi;
       if not (List.mem name pinstr_names) then
-	pinstr_names <- name :: pinstr_names;
+	pinstr_names <- pinstr_names @ [name];
 
 
     method element name =
@@ -655,11 +658,12 @@ and dtd_element the_dtd the_name =
     method arbitrary_allowed = allow_arbitrary
 
     method add_attribute aname t d extdecl =
+      let swarner = dtd#swarner 
+      and warner = dtd#warner in
       if aname <> "xml:lang" & aname <> "xml:space" then
-	check_name (dtd#warner) aname;
+	check_name ?swarner warner aname;
       if List.mem_assoc aname attributes then
-	dtd # warner # warn ("More than one declaration for attribute `" ^
-			     aname ^ "' of element type `" ^ name ^ "'")
+	warn swarner warner (`W_multiple_attribute_declarations(name,aname))
       else begin
 	begin match aname with
 	    "xml:space" ->
@@ -1042,7 +1046,8 @@ and dtd_element the_dtd the_name =
        *)
       match content_model with
 	  Unspecified ->
-	    dtd # warner # warn ("Element type `" ^ name ^ "' mentioned but not declared");
+	    warn (dtd#swarner) (dtd#warner)
+ 	         (`W_element_mentioned_but_not_declared name);
 	    ()
 	| Empty -> ()
 	| Any -> ()
@@ -1064,7 +1069,7 @@ and dtd_notation the_name the_xid init_encoding =
 object (self)
     val name = the_name
     val xid = (the_xid : ext_id)
-    val encoding = (init_encoding : Pxp_types.rep_encoding)
+    val encoding = (init_encoding : Pxp_core_types.rep_encoding)
     method name = name
     method ext_id = xid
     method encoding = encoding
@@ -1105,7 +1110,7 @@ and proc_instruction the_target the_value init_encoding =
 object (self)
     val target = the_target
     val value = (the_value : string)
-    val encoding = (init_encoding : Pxp_types.rep_encoding)
+    val encoding = (init_encoding : Pxp_core_types.rep_encoding)
 
     initializer
       match target with
@@ -1151,6 +1156,7 @@ object (self)
 type source =
     Entity of ((dtd -> Pxp_entity.entity) * Pxp_reader.resolver)
   | ExtID of (ext_id * Pxp_reader.resolver)
+  | XExtID of (ext_id * string option * Pxp_reader.resolver)
 ;;
 
 module Entity = struct
@@ -1163,22 +1169,30 @@ module Entity = struct
   let replacement_text ent = fst(ent # replacement_text)
   let get_xid ent =
     try Some(ent # ext_id) with Not_found -> None
+  let get_resolver_id ent =
+    try Some(ent # resolver_id) with Not_found -> None
   let get_notation ent =
     if ent # is_ndata then Some (ent # notation) else None
   let create_internal_entity ~name ~value dtd =
-    new internal_entity dtd name (dtd # warner) value false false 
-        (dtd # encoding)
+    new internal_entity dtd name (dtd # swarner) (dtd # warner) value 
+        false false (dtd # encoding)
   let create_ndata_entity ~name ~xid ~notation dtd =
     new ndata_entity name xid notation dtd#encoding
-  let create_external_entity ?(doc_entity = false) ~name ~xid ~resolver dtd =
+  let create_external_entity ?(doc_entity = false) ?system_base 
+                             ~name ~xid ~resolver dtd =
     if doc_entity then
-      new document_entity resolver dtd name dtd#warner xid dtd#encoding
+      new document_entity resolver dtd name dtd#swarner dtd#warner xid
+	                  system_base dtd#encoding
     else
-      new external_entity resolver dtd name dtd#warner xid false dtd#encoding
+      new external_entity resolver dtd name dtd#swarner dtd#warner xid
+	                  system_base false dtd#encoding
   let from_external_source ?doc_entity ~name dtd src =
     match src with
 	ExtID(xid,resolver) -> 
 	  create_external_entity ?doc_entity ~name ~xid ~resolver dtd
+      | XExtID(xid,system_base,resolver) -> 
+	  create_external_entity ?doc_entity ?system_base 
+                                 ~name ~xid ~resolver dtd
       | Entity(make,resolver) ->
 	  make dtd  (* resolver ignored *)
 end
@@ -1188,6 +1202,20 @@ end
  * History:
  *
  * $Log: pxp_dtd.ml,v $
+ * Revision 1.25  2003/06/20 15:14:13  gerd
+ * 	Introducing symbolic warnings, expressed as polymorphic
+ * variants
+ *
+ * Revision 1.24  2003/06/15 12:23:21  gerd
+ * 	Moving core type definitions to Pxp_core_types
+ *
+ * Revision 1.23  2003/03/23 21:43:37  gerd
+ * 	Preserving the correct order of processing instructions
+ * (Blair Zajac)
+ *
+ * Revision 1.22  2003/01/21 00:19:18  gerd
+ * 	Support for resolver_id.
+ *
  * Revision 1.21  2002/03/10 23:39:28  gerd
  * 	Extended the Entity module
  *
