@@ -1,11 +1,11 @@
-(* $Id: pxp_entity.ml,v 1.16 2002/03/10 23:39:45 gerd Exp $
+(* $Id: pxp_entity.ml,v 1.24 2003/11/04 22:45:07 gerd Exp $
  * ----------------------------------------------------------------------
  * PXP: The polymorphic XML parser for Objective Caml.
  * Copyright by Gerd Stolpmann. See LICENSE for details.
  *)
 
 
-open Pxp_types
+open Pxp_core_types
 open Pxp_lexer_types
 open Pxp_aux
 open Pxp_reader
@@ -72,6 +72,7 @@ end
 type 'entity entity_variables = 
     { mutable dtd : 'entity preliminary_dtd;
       mutable name : string;
+      mutable swarner : symbolic_warnings option;
       mutable warner : collect_warnings;
       
       mutable encoding : rep_encoding;
@@ -82,9 +83,9 @@ type 'entity entity_variables =
       mutable lexerset_scan_content : Lexing.lexbuf -> (token * lexers);
       mutable lexerset_scan_within_tag : Lexing.lexbuf -> (token * lexers);
       mutable lexerset_scan_declaration : Lexing.lexbuf -> (token * lexers);
-      mutable lexerset_scan_decl_comment : Lexing.lexbuf -> (token * lexers);
-      mutable lexerset_scan_content_comment : Lexing.lexbuf -> (token * lexers);
-      mutable lexerset_scan_document_comment : Lexing.lexbuf -> (token * lexers);
+      mutable lexerset_scan_comment : Lexing.lexbuf -> lexers -> (token * lexers);
+      mutable lexerset_scan_tag_eb : Lexing.lexbuf -> (token * lexers);
+      mutable lexerset_scan_tag_eb_att : Lexing.lexbuf -> bool -> (token * lexers);
       mutable lexbuf : Lexing.lexbuf;
         (* The lexical buffer currently used as character source. *)
 
@@ -112,6 +113,9 @@ type 'entity entity_variables =
       mutable normalize_newline : bool;
         (* Whether this entity converts CRLF or CR to LF, or not *)
 
+      mutable generate_attribute_events : bool;
+        (* Whether attribute events are generated or not *)
+
       mutable line : int;          (* current line *)
       mutable column : int;        (* current column *)
 
@@ -136,11 +140,12 @@ type 'entity entity_variables =
 ;;
 
 
-let make_variables the_dtd the_name the_warner init_encoding =
+let make_variables the_dtd the_name the_swarner the_warner init_encoding =
   let ls = Pxp_lexers.get_lexer_set init_encoding in
   { dtd = (the_dtd : 'entity #preliminary_dtd :> 'entity preliminary_dtd);
     name = the_name;
     warner = the_warner;
+    swarner = the_swarner;
     
     encoding = init_encoding;
     lexerset = ls;
@@ -150,9 +155,9 @@ let make_variables the_dtd the_name the_warner init_encoding =
     lexerset_scan_content          = ls.scan_content;
     lexerset_scan_within_tag       = ls.scan_within_tag;
     lexerset_scan_declaration      = ls.scan_declaration;
-    lexerset_scan_decl_comment     = ls.scan_decl_comment;
-    lexerset_scan_content_comment  = ls.scan_content_comment;
-    lexerset_scan_document_comment = ls.scan_document_comment;
+    lexerset_scan_comment          = ls.scan_comment;
+    lexerset_scan_tag_eb           = ls.scan_tag_eb;
+    lexerset_scan_tag_eb_att       = ls.scan_tag_eb_att;
     
     lexbuf = Pxp_lexing.from_string "";
     
@@ -165,7 +170,8 @@ let make_variables the_dtd the_name the_warner init_encoding =
     check_text_declaration = true;
     
     normalize_newline = true;
-    
+    generate_attribute_events = false;
+
     line = 1;
     column = 0;
 
@@ -199,8 +205,7 @@ let update_content_lines v tok =
 	count_lines v.linecount (Lexing.lexeme v.lexbuf);
 	update_lines v;
     | _ -> 
-	v.column <- v.column + Lexing.lexeme_end v.lexbuf 
-	                     - Lexing.lexeme_start v.lexbuf
+	v.column <- v.column + Pxp_lexing.lexeme_len v.lexbuf
 ;;
 
 let update_lines_within_tag v tok =
@@ -218,8 +223,7 @@ let update_lines_within_tag v tok =
 	v.line <- v.line + 1;
 	v.column <- 0;
     | _ -> 
-	v.column <- v.column + Lexing.lexeme_end v.lexbuf 
-	                     - Lexing.lexeme_start v.lexbuf
+	v.column <- v.column + Pxp_lexing.lexeme_len v.lexbuf 
 ;;
 
 let update_other_lines v tok =
@@ -228,24 +232,31 @@ let update_other_lines v tok =
 ;;
 
 
-class virtual entity the_dtd the_name the_warner init_encoding =
+class virtual entity the_dtd the_name the_swarner the_warner init_encoding =
   object (self)
     (* This class prescribes the type of all entity objects. Furthermore,
      * the default 'next_token' mechanism is implemented.
      *)
 
-    val v = make_variables the_dtd the_name the_warner init_encoding
+    val v = make_variables 
+	      the_dtd the_name the_swarner the_warner init_encoding
 
     method is_ndata = false
       (* Returns if this entity is an NDATA (unparsed) entity *)
 
     method name = v.name
 
+    method lex_id = v.lex_id
+
     method set_lex_id id = v.lex_id <- id
 
     method line = v.p_line
     method column = v.p_column
 
+    method set_line_column l c =
+      v.line <- l;
+      v.column <- c
+
     method encoding = v.encoding
     (* method lexerset = lexerset *)
 
@@ -257,7 +268,7 @@ class virtual entity the_dtd the_name the_warner init_encoding =
 	    None -> assert false
 	  | Some m -> m
       : < current_entity : entity; 
-	  pop_entity : unit;
+	  pop_entity : unit -> unit;
 	  push_entity : entity -> unit >
       )
 
@@ -268,8 +279,12 @@ class virtual entity the_dtd the_name the_warner init_encoding =
     method set_counts_as_external =
       v.counts_as_external <- true
 
+    method lexbuf = v.lexbuf
 
-    method virtual open_entity : bool -> lexers -> unit
+    method virtual resolver : resolver option
+
+    method virtual open_entity : 
+        ?gen_att_events:bool -> bool -> lexers -> unit
 	(* open_entity force_parsing lexid:
 	 * opens the entity, and the first token is scanned by the lexer
 	 * 'lexid'. 'force_parsing' forces that Begin_entity and End_entity
@@ -298,11 +313,19 @@ class virtual entity the_dtd the_name the_warner init_encoding =
 	 * from non-document entities do not add these tokens.
 	 *)
 
-    method virtual close_entity : lexers
+    method close_entity =
 	(* close_entity:
 	 * closes the entity and returns the name of the lexer that must
 	 * be used to scan the next token.
 	 *)
+      let current_lex_id = v.lex_id in
+      v.deferred_token <- None;
+      v.lex_id <- Closed;
+      current_lex_id
+
+    method virtual is_open : bool
+      (* Whether the entity is currently open *)
+
 
     method virtual replacement_text : (string * bool)
 	(* replacement_text:
@@ -361,35 +384,48 @@ class virtual entity the_dtd the_name the_warner init_encoding =
 	     * name lex_id' of the next lexer to be used.
 	     *)
 	    let update_fn = ref update_content_lines in
-	    let scan_fn =
+	    let tok, lex_id' =
 	      match v.lex_id with
 		  Document         -> update_fn := update_other_lines;
-                                      v.lexerset_scan_document
+                                      v.lexerset_scan_document v.lexbuf 
 		| Document_type    -> update_fn := update_other_lines;
-                                      v.lexerset_scan_document_type
-		| Content          -> v.lexerset_scan_content
+                                      v.lexerset_scan_document_type v.lexbuf
+		| Content          -> v.lexerset_scan_content v.lexbuf
 		| Within_tag       -> update_fn := update_lines_within_tag;
-                                      v.lexerset_scan_within_tag
+                                      v.lexerset_scan_within_tag v.lexbuf
+		| Within_tag_entry -> if v.generate_attribute_events then (
+		                        (* like Tag_eb: *)
+		                        update_fn := update_lines_within_tag;
+                                        v.lexerset_scan_tag_eb v.lexbuf
+                                      ) else (
+		                        (* like Within_tag: *)
+		                        update_fn := update_lines_within_tag;
+                                        v.lexerset_scan_within_tag v.lexbuf
+		                      )
 		| Declaration      -> update_fn := update_other_lines;
-                                      v.lexerset_scan_declaration
-		| Content_comment  -> update_fn := update_other_lines;
-                                      v.lexerset_scan_content_comment
-		| Decl_comment     -> update_fn := update_other_lines;
-                                      v.lexerset_scan_decl_comment
-		| Document_comment -> update_fn := update_other_lines;
-                                      v.lexerset_scan_document_comment
+                                      v.lexerset_scan_declaration v.lexbuf
+		| Comment flw_id   -> update_fn := update_other_lines;
+		                      v.lexerset_scan_comment v.lexbuf flw_id
+		| Tag_eb           -> update_fn := update_lines_within_tag;
+                                      v.lexerset_scan_tag_eb v.lexbuf
+		| Tag_eb_att b     -> (* keep update_content_lines! *)
+                                      v.lexerset_scan_tag_eb_att v.lexbuf b
 		| Ignored_section  -> assert false
-  	          (* Ignored_section: only used by method next_ignored_token *)
+  	                           (* only used by method next_ignored_token *)
+		| Closed           -> (Eof, Closed)
 	    in
-	    let tok, lex_id' = scan_fn v.lexbuf in
 
+	    if debug then (
+	      prerr_endline ("- Entity " ^ v.name ^ ": " ^ string_of_tok tok);
+	      prerr_endline ("         Transition: " ^ 
+			     string_of_lexers v.lex_id ^ " -> " ^ 
+			     string_of_lexers lex_id')
+	    );
+	    
 	    (* Find out the number of lines and characters of the last line: *)
 	    !update_fn v tok;
 	    v.lex_id <- lex_id';
 
-	    if debug then
-	      prerr_endline ("- Entity " ^ v.name ^ ": " ^ string_of_tok tok);
-	    
 	    (* Throw Ignore and Comment away; Interpret entity references: *)
 	    (* NOTE: Of course, references to general entities are not allowed
 	     * everywhere; parameter references, too. This is already done by the
@@ -410,13 +446,17 @@ class virtual entity the_dtd the_name the_warner init_encoding =
 			   ("Reference to entity `" ^ n ^ 
 			    "' violates standalone declaration"));
 		    en # set_debugging_mode debug;
-	            en # open_entity true v.lex_id;
+	            en # open_entity 
+                      ?gen_att_events:(Some v.generate_attribute_events) 
+                      true v.lex_id;
 		    self # manager # push_entity en;
 		    en # next_token;
 		| PERef n   -> 
 		    let en = v.dtd # par_entity n in
 		    en # set_debugging_mode debug;
-	            en # open_entity v.force_parameter_entity_parsing v.lex_id;
+	            en # open_entity 
+		      ?gen_att_events:(Some v.generate_attribute_events)
+                      v.force_parameter_entity_parsing v.lex_id;
 		    self # manager # push_entity en;
 		    en # next_token;
 
@@ -427,6 +467,13 @@ class virtual entity the_dtd the_name the_warner init_encoding =
 		    else
 		      CharData s
 
+          (* Convert LineEnd_att to CharData *)
+		| LineEnd_att s -> 
+		    if v.normalize_newline then 
+		      CharData " "
+		    else
+		      CharData s
+
           (* Also normalize CDATA sections *)
 		| Cdata value as cd ->
 		    if v.normalize_newline then 
@@ -536,7 +583,7 @@ class virtual entity the_dtd the_name the_warner init_encoding =
     method private handle_eof =
       let mng = self # manager in
       begin try
-	mng # pop_entity;
+	mng # pop_entity();
 	let next_lex_id = self # close_entity in
 	let en = mng # current_entity in
 	en # set_lex_id next_lex_id;
@@ -553,14 +600,17 @@ class virtual entity the_dtd the_name the_warner init_encoding =
 
       (* TODO: Do we need a test on deferred tokens here? *)
 
-	let tok, lex_id' = v.lexerset.scan_ignored_section v.lexbuf in
-	if v.debug then
-	  prerr_endline ("- Entity " ^ v.name ^ ": " ^ string_of_tok tok ^ " (Ignored)");
-	update_other_lines v tok;
-	match tok with
-	  | Conditional_begin _ -> Conditional_begin (self :> entity_id)
-	  | Conditional_end _   -> Conditional_end   (self :> entity_id)
-	  | _                   -> tok
+        if v.lex_id = Closed then
+	  Eof
+	else
+	  let tok, lex_id' = v.lexerset.scan_ignored_section v.lexbuf in
+	  if v.debug then
+	    prerr_endline ("- Entity " ^ v.name ^ ": " ^ string_of_tok tok ^ " (Ignored)");
+	  update_other_lines v tok;
+	  match tok with
+	    | Conditional_begin _ -> Conditional_begin (self :> entity_id)
+	    | Conditional_end _   -> Conditional_end   (self :> entity_id)
+	    | _                   -> tok
 
 
     method process_xmldecl pl =
@@ -598,6 +648,13 @@ class virtual entity the_dtd the_name the_warner init_encoding =
       (raise Not_found : ext_id)
 
 
+    method resolver_id =
+      (* Returns the resolver ID for external entities. Raises Not_found
+       * for other types of entities.
+       *)
+      (raise Not_found : resolver_id)
+
+
     (* Methods for NDATA entities only: *)
     method notation = (assert false : string)
 
@@ -619,6 +676,7 @@ class ndata_entity the_name the_ext_id the_notation init_encoding =
     method name = (name : string)
     method ext_id = (ext_id : ext_id)
     method notation = (notation : string)
+    method resolver_id = (raise Not_found : resolver_id)
 
     method is_ndata = true
 
@@ -635,11 +693,17 @@ class ndata_entity the_name the_ext_id the_notation init_encoding =
 
 
     method set_manager (m : < current_entity : entity; 
-			      pop_entity : unit;
+			      pop_entity : unit -> unit;
 			      push_entity : entity -> unit >) = 
       ( raise (Validation_error ("Invalid reference to NDATA entity " ^ name))
 	  : unit )
 
+    method resolver = (None : resolver option)
+
+    method lex_id =
+      ( raise (Validation_error ("Invalid reference to NDATA entity " ^ name))
+	  : lexers)
+
     method set_lex_id (id : lexers) =
       ( raise (Validation_error ("Invalid reference to NDATA entity " ^ name))
 	  : unit )
@@ -652,6 +716,10 @@ class ndata_entity the_name the_ext_id the_notation init_encoding =
       ( raise (Validation_error ("Invalid reference to NDATA entity " ^ name))
 	  : int )
 
+    method set_line_column (_:int) (_:int) =
+      ( raise (Validation_error ("Invalid reference to NDATA entity " ^ name))
+	  : unit )
+
     method full_name =
       ( raise (Validation_error ("Invalid reference to NDATA entity " ^ name))
 	  : string )
@@ -663,7 +731,11 @@ class ndata_entity the_name the_ext_id the_notation init_encoding =
 
     method set_debugging_mode (_:bool) = ()
 
-    method open_entity (_:bool) (_:lexers) =
+    method lexbuf =
+      ( raise (Validation_error ("Invalid reference to NDATA entity " ^ name))
+	  : Lexing.lexbuf )
+
+    method open_entity ?(gen_att_events:bool option) (_:bool) (_:lexers) =
       ( raise (Validation_error ("Invalid reference to NDATA entity " ^ name))
 	  : unit )
 
@@ -671,6 +743,8 @@ class ndata_entity the_name the_ext_id the_notation init_encoding =
       ( raise (Validation_error ("Invalid reference to NDATA entity " ^ name))
 	  : lexers )
 
+    method is_open = false  (* NDATA entities cannot be opened *)
+
     method replacement_text =
       ( raise (Validation_error ("Invalid reference to NDATA entity " ^ name))
 	  : (string * bool) )
@@ -698,13 +772,15 @@ class ndata_entity the_name the_ext_id the_notation init_encoding =
   end
 ;;
 
-class external_entity the_resolver the_dtd the_name the_warner the_ext_id
+class external_entity the_resolver the_dtd the_name the_swarner the_warner 
+                      the_ext_id
+                      the_system_base
                       the_p_special_empty_entities
 		      init_encoding
   =
   object (self)
     inherit entity
-              the_dtd the_name the_warner 
+              the_dtd the_name the_swarner the_warner 
 	      init_encoding
             as super
 
@@ -721,7 +797,15 @@ class external_entity the_resolver the_dtd the_name the_warner the_ext_id
      *)
 
     val resolver = (the_resolver : resolver)
-    val ext_id = (the_ext_id : ext_id)
+    val ext_id = 
+      match (the_ext_id : ext_id) with
+	  Public(pubid,sysid) ->
+	    (* normalize pubid: *)
+	    Public(Pxp_aux.normalize_public_id pubid,sysid)
+	| other ->
+	    other
+
+    val system_base = (the_system_base : string option)
 
     val p_special_empty_entities = (the_p_special_empty_entities : bool)
 
@@ -751,7 +835,13 @@ class external_entity the_resolver the_dtd the_name the_warner the_ext_id
 
     method ext_id = ext_id
 
-    method open_entity force_parsing init_lex_id =
+    method resolver = Some resolver
+
+    method resolver_id = 
+      let rid = resolver_id_of_ext_id ext_id in
+      { rid with rid_system_base = system_base }
+
+    method open_entity ?(gen_att_events=false) force_parsing init_lex_id =
       (* Note that external entities are always parsed, i.e. Begin_entity
        * and End_entity tokens embrace the inner tokens to force that
        * the entity is only called where the syntax allows it.
@@ -760,7 +850,7 @@ class external_entity the_resolver the_dtd the_name the_warner the_ext_id
 	raise(Validation_error("Recursive reference to entity `" ^ v.name ^ "'"));
       let lex = 
 	try
-	  resolver # open_in ext_id 
+	  resolver # open_rid (self # resolver_id)
 	with
 	    Pxp_reader.Not_competent ->
 	      raise(Error ("No input method available for this external entity: " ^ 
@@ -782,6 +872,7 @@ class external_entity the_resolver the_dtd the_name the_warner the_ext_id
       v.column <- 0;
       v.at_bof <- true;
       v.normalize_newline <- true;
+      v.generate_attribute_events <- gen_att_events;
 
 
     method private handle_bof tok =
@@ -824,7 +915,10 @@ class external_entity the_resolver the_dtd the_name the_warner the_ext_id
 	failwith ("External entity " ^ v.name ^ " not open");
       resolver # close_in;
       resolver_is_open <- false;
-      v.lex_id
+      super # close_entity
+
+
+    method is_open = resolver_is_open
 
 
     method replacement_text =
@@ -838,7 +932,7 @@ class external_entity the_resolver the_dtd the_name the_warner the_ext_id
 	raise(Validation_error("Recursive reference to entity `" ^ v.name ^ "'"));
       let lex = 
 	try
-	  resolver # open_in ext_id 
+	  resolver # open_rid (self # resolver_id)
 	with
 	    Pxp_reader.Not_competent ->
 	      raise(Error ("No input method available for this external entity: " ^ 
@@ -860,18 +954,28 @@ class external_entity the_resolver the_dtd the_name the_warner the_ext_id
       v.column <- 0;
       v.at_bof <- true;
       (* First check if the first token of 'lex' is <?xml...?> *)
-      begin match v.lexerset.scan_only_xml_decl lex with
-	  PI_xml pl ->
-	    self # process_xmldecl pl
-	| Eof ->
-	    (* This only means that the first token was not <?xml...?>;
-	     * the "Eof" token represents the empty string.
-	     *)
-	    self # process_missing_xmldecl
-	| _ ->
-	    (* Must not happen. *)
-	    assert false
-      end;
+      if v.lexerset.detect_xml_pi lex then begin
+	(* detect_xml_pi scans "<?xml" ws+. Read the rest of the XML
+	 * declaration.
+	 *)
+	match v.lexerset.scan_pi_string lex with
+	    Some pi ->
+	      begin match
+		Pxp_lex_aux.scan_pi ("xml " ^ pi) v.lexerset.scan_xml_pi
+	      with
+		  PI_xml pl ->
+		    self # process_xmldecl pl
+		| _ ->
+		    assert false   (* cannot happen *)
+	      end
+	  | None ->
+	      raise(WF_error("Bad XML declaration"))
+      end
+      else
+	(* This only means that the first token was not <?xml...?>;
+	 * the "Eof" token represents the empty string.
+	 *)
+	self # process_missing_xmldecl;
       (* Then create the replacement text. *)
       let rec scan_and_expand () =
 	match v.lexerset.scan_dtd_string v.lexbuf with
@@ -879,7 +983,9 @@ class external_entity the_resolver the_dtd the_name the_warner the_ext_id
 	  | CRef(-1) -> "\n" ^ scan_and_expand()
 	  | CRef(-2) -> "\n" ^ scan_and_expand()
 	  | CRef(-3) -> "\n" ^ scan_and_expand()
-	  | CRef k -> character v.encoding v.warner k ^ scan_and_expand()
+	  | CRef k -> 
+	      character 
+	        ?swarner:v.swarner v.encoding v.warner k ^ scan_and_expand()
 	  | CharData x -> x ^ scan_and_expand()
 	  | PERef n ->
 	      let en = v.dtd # par_entity n in
@@ -901,12 +1007,15 @@ class external_entity the_resolver the_dtd the_name the_warner the_ext_id
 ;;
 
 
-class document_entity  the_resolver the_dtd the_name the_warner the_ext_id
+class document_entity  the_resolver the_dtd the_name the_swarner the_warner
+                       the_ext_id
+                       the_system_base
 		       init_encoding
   =
   object (self)
-    inherit external_entity  the_resolver the_dtd the_name the_warner
-                             the_ext_id false 
+    inherit external_entity  the_resolver the_dtd the_name the_swarner 
+                             the_warner
+                             the_ext_id the_system_base false 
 			     init_encoding
 
     (* A document entity is an external entity that does not allow
@@ -924,7 +1033,7 @@ class document_entity  the_resolver the_dtd the_name the_warner the_ext_id
 ;;
 
 
-class internal_entity the_dtd the_name the_warner the_literal_value
+class internal_entity the_dtd the_name the_swarner the_warner the_literal_value
                       the_p_internal_subset 
                       init_is_parameter_entity
 		      init_encoding
@@ -941,7 +1050,7 @@ class internal_entity the_dtd the_name the_warner the_literal_value
 
   object (self)
     inherit entity
-              the_dtd the_name the_warner 
+              the_dtd the_name the_swarner the_warner 
 	      init_encoding
 	    as super
 
@@ -963,7 +1072,9 @@ class internal_entity the_dtd the_name the_warner the_literal_value
 	| CRef(-1) -> "\r\n" ^ scan_and_expand()
 	| CRef(-2) -> "\r" ^ scan_and_expand()
 	| CRef(-3) -> "\n" ^ scan_and_expand()
-	| CRef k -> character v.encoding v.warner k ^ scan_and_expand()
+	| CRef k -> 
+	    character 
+	      ?swarner:v.swarner v.encoding v.warner k ^ scan_and_expand()
 	| CharData x -> x ^ scan_and_expand()
 	| PERef n ->
 	    if p_internal_subset then
@@ -998,7 +1109,7 @@ class internal_entity the_dtd the_name the_warner the_literal_value
       assert(e = "");
 
 
-    method open_entity force_parsing init_lex_id =
+    method open_entity ?(gen_att_events = false) force_parsing init_lex_id =
       if is_open then
 	raise(Validation_error("Recursive reference to entity `" ^ v.name ^ "'"));
 
@@ -1015,6 +1126,7 @@ class internal_entity the_dtd the_name the_warner the_literal_value
       v.line <- 1;
       v.column <- 0;
       v.at_bof <- true;       (* CHECK: Is this right? *)
+      v.generate_attribute_events <- gen_att_events;
 
 
     method private handle_bof tok =
@@ -1062,121 +1174,178 @@ class internal_entity the_dtd the_name the_warner the_literal_value
       if not is_open then
 	failwith ("Internal entity " ^ v.name ^ " not open");
       is_open <- false;
-      v.lex_id
+      super # close_entity
+
+
+    method is_open = is_open
 
 
     method replacement_text =
       if is_open then
 	raise(Validation_error("Recursive reference to entity `" ^ v.name ^ "'"));
       replacement_text, contains_external_references
+
+
+    method resolver = (None : resolver option)
   end
 ;;
 
-(**********************************************************************)
 
-(* An 'entity_manager' is a stack of entities, where the topmost entity
- * is the currently active entity, the second entity is the entity that
- * referred to the active entity, and so on.
- *
- * The entity_manager can communicate with the currently active entity.
- *
- * The entity_manager provides an interface for the parser; the functions
- * returning the current token and the next token are exported.
+(* An entity_section is an object that reads a section from an underlying
+ * entity as if this section was an entity of its own. In detail, the
+ * following rules apply:
+ * - If a token is read from the entity_section, it is actually read from
+ *   the underlying entity (except the first and the last token). I.e.
+ *   the token stream of the entity_section and the underlying entity is
+ *   essentially the same.
+ * - However, the entity_section has its own lexical context. The method
+ *   set_lex_id changes only the lexer ID of the entity_section, and not
+ *   of the underlying entity.
+ * - The first token is always Begin_entity, and the last token is always
+ *   End_entity. These tokens are not taken from the underlying entity,
+ *   but simpy pretended at the beginning and at the end of the section.
+ * - The section begins at the current position of the (open) underlying
+ *   entity when the method open_entity of the section is called. It is 
+ *   an error if the underlying entity is at the beginning itself.
+ *   [TODO: The latter condition is currently not checked.]
+ * - The section ends when the method close_entity is called. The next
+ *   token will be End_token, and then an endless sequence of Eof.
+ * - A section cannot be opened a second time.
+ * - Changes of encodings are ignored. (The underlying entity must do that.)
  *)
 
-class entity_manager (init_entity : entity) =
-  object (self)
-    val mutable entity_stack = Stack.create()
-    val mutable current_entity = init_entity
-    val mutable current_entity's_full_name = lazy (init_entity # full_name)
-				   
-    val mutable yy_get_next_ref = ref (fun () -> assert false)
+type section_state = P_bof | P_normal of int | P_pre_eof | P_eof
+  (* P_normal n: The number n is the number of open inner entities *)
 
-    initializer
-      init_entity # set_manager (self :> 
-				 < current_entity : entity; 
-				   pop_entity : unit;
-				   push_entity : entity -> unit >
-				);
-      yy_get_next_ref := (fun () -> init_entity # next_token)
+class entity_section (init_ent:entity) =
+object (self) 
+  val ent = init_ent
+  val mutable state = P_bof
+  val mutable is_open = false
+  val mutable saved_lex_id = Closed
 
-    method push_entity e =
-      e # set_manager (self :> 
-		       < current_entity : entity; 
-		         pop_entity : unit;
-			 push_entity : entity -> unit >
-		      );
-      Stack.push (current_entity, current_entity's_full_name) entity_stack;
-      current_entity <- e;
-      current_entity's_full_name <- lazy (e # full_name);
-      yy_get_next_ref := (fun () -> e # next_token);
-
-    method pop_entity =
-      (* May raise Stack.Empty *)
-      let e, e_name = Stack.pop entity_stack in
-      current_entity <- e;
-      current_entity's_full_name <- e_name;
-      yy_get_next_ref := (fun () -> e # next_token);
-
-
-
-    method position_string =
-      (* Gets a string describing the position of the last token;
-       * includes an entity backtrace
-       *)
-      let b = Buffer.create 200 in
-      Buffer.add_string b
-	("In entity " ^ current_entity # full_name
-	 ^ ", at line " ^ string_of_int (current_entity # line)
-	 ^ ", position " ^ string_of_int (current_entity # column)
-	 ^ ":\n");
-      Stack.iter
-	(fun (e, e_name) ->
-	   Buffer.add_string b 
-	     ("Called from entity " ^ Lazy.force e_name
-	      ^ ", line " ^ string_of_int (e # line)
-	      ^  ", position " ^ string_of_int (e # column)
-	      ^ ":\n");
-	)
-	entity_stack;
-      Buffer.contents b
-
-
-    method position =
-      (* Returns the triple (full_name, line, column) of the last token *)
-      Lazy.force current_entity's_full_name, 
-      current_entity # line,
-      current_entity # column
-
-
-    method current_entity_counts_as_external =
-      (* Whether the current entity counts as external to the main
-       * document for the purpose of stand-alone checks.
-       *)
-      (* TODO: improve performance *)
-      let is_external = ref false in
-      let check (e, _) =
-	if e # counts_as_external then begin
-	  is_external := true;
-	end;
-      in
-      check (current_entity,());
-      Stack.iter check entity_stack;
-      !is_external
-
-
-    method current_entity  = current_entity
-
-    method yy_get_next_ref = yy_get_next_ref
-
-  end
+  method is_ndata = ent # is_ndata
+  method name = ent # name
+  method lex_id = ent # lex_id
+  method set_lex_id = 
+    if not is_open then
+      failwith "Pxp_entity.entity_section#set_lex_id: not open";
+    ent # set_lex_id
+  method line = ent # line
+  method column = ent # column
+  method set_line_column = 
+    if not is_open then
+      failwith "Pxp_entity.entity_section#set_line_column: not open";
+    ent # set_line_column
+  method encoding = ent # encoding
+  method set_manager (_ : < current_entity : entity; 
+		            pop_entity : unit -> unit;
+			    push_entity : entity -> unit >) = ()
+  method counts_as_external = ent # counts_as_external
+  method set_counts_as_external : unit = 
+    failwith "Pxp_entity.entity_section#set_counts_as_external: not possible";
+  method lexbuf = ent # lexbuf
+  method resolver = ent # resolver
+  method resolver_id = ent # resolver_id
+  method open_entity ?(gen_att_events:bool option) (_:bool) (lid:lexers) = 
+    if is_open then
+      failwith "Pxp_entity.entity_section#open_entity: already open";
+    if not ent#is_open then
+      failwith "Pxp_entity.entity_section#open_entity: Underlying entity is not open";
+    saved_lex_id <- ent # lex_id;
+    state <- P_bof;
+    is_open <- true;
+    ent # set_lex_id lid;
+  method close_entity =
+    if not is_open then
+      failwith "Pxp_entity.entity_section#close_entity: not open";
+    is_open <- false;
+    ent # set_lex_id saved_lex_id;
+    assert (match state with P_bof | P_normal _ -> true | _ -> false);
+    (* CHECK: P_normal n when n>0 *)
+    state <- P_pre_eof;
+    saved_lex_id
+  method is_open = is_open
+  method replacement_text : (string * bool) =
+    failwith "Pxp_entity.entity_section#replacement_text: not possible"
+  method xml_declaration = ent # xml_declaration
+  method set_debugging_mode = ent # set_debugging_mode
+  method full_name = ent # full_name
+  method next_token =
+    match state with
+      | P_bof ->
+	  state <- P_normal 0;
+	  Begin_entity
+      | P_normal n -> 
+	  let tok = ent # next_token in
+	  (* Notes:
+	   * - [tok = Begin_entity] can have two reasons: [ent] has produced
+	   *   [Begin_entity], or [ent] has just found an entity reference
+	   *   whose entity has been opened. Because the latter is possible
+	   *   we do not catch [Begin_entity] here.
+	   * - [tok = End_entity]: This token is always produced by [ent],
+	   *   and so we can signal an error 
+	   *)
+	  ( match tok with
+	      | (End_entity | Eof) ->
+		  raise(Error "Cannot end entity here")
+	      | _ ->
+		  tok
+	  )
+      | P_pre_eof -> 
+	  state <- P_eof; 
+	  End_entity
+      | P_eof -> 
+	  Eof
+  method next_ignored_token =
+    (* We can ignore End_entity and Eof because the caller already signals
+     * an error when the entity ends in an IGNORE section
+     *)
+    ent # next_ignored_token
+  method process_xmldecl (_:prolog_token list) = ()
+  method process_missing_xmldecl = ()
+  method ext_id = ent # ext_id
+  method notation = ent # notation
+end
 ;;
-      
+
+(* class entity_manager: has been moved to Pxp_entity_manager *)
 
 (* ======================================================================
  * History:
  *
  * $Log: pxp_entity.ml,v $
+ * Revision 1.24  2003/11/04 22:45:07  gerd
+ * 	Problems with Lexing under 3.07 fixed
+ *
+ * Revision 1.23  2003/06/20 15:14:14  gerd
+ * 	Introducing symbolic warnings, expressed as polymorphic
+ * variants
+ *
+ * Revision 1.22  2003/06/15 12:23:21  gerd
+ * 	Moving core type definitions to Pxp_core_types
+ *
+ * Revision 1.21  2003/01/21 00:19:05  gerd
+ * 	Support for resolver_id.
+ *
+ * Revision 1.20  2002/08/31 23:23:43  gerd
+ * 	Replaced the three comment lexers by only one unified comment
+ * lexer.
+ * 	Improved way of detecting the XML declaration at the beginning
+ * of external files when computing the replacement text.
+ *
+ * Revision 1.19  2002/08/05 22:33:33  gerd
+ * 	New token LineEnd_att for newline characters inside
+ * attribute values (event-based parser).
+ *
+ * Revision 1.18  2002/08/03 17:52:34  gerd
+ * 	Support for event-based parsing of attribute values. There is
+ * now a flag gen_att_events that determines whether to use this feature.
+ * 	Fix in open_entity of class entity_section: Sets the lex_id.
+ *
+ * Revision 1.17  2002/07/14 23:03:29  gerd
+ * 	New class entity_section.
+ *
  * Revision 1.16  2002/03/10 23:39:45  gerd
  * 	ext_id works also for external entities.
  *
